@@ -4,16 +4,21 @@ use 5.010;
 use strict;
 use warnings;
 
-use base 'Exporter';
-our @EXPORT_OK = qw/pkg/;
-
+use base 'Csistck::Test';
 use Csistck::Oper qw/debug/;
 use Csistck::Config qw/option/;
-use Csistck::Test;
+
+our @EXPORT_OK = qw/pkg/;
 
 use Digest::MD5;
 use File::Basename;
-use Linux::Distribution qw/distribution_name/;
+
+# Conditionally use linux-only modules
+BEGIN {
+    if ("$^O" eq "linux") {
+        require Linux::Distribution;
+    }
+}
 
 our $Cmds = {
     dpkg => {
@@ -33,6 +38,9 @@ our $Cmds = {
     pacman => {
         check => 'pacman -Qe "%s"',
         install => 'pacman -Sq --noconfirm "%s"'
+    },
+    pkg_info => {
+        check => 'pkg_info -Qq "%s>0"'
     }
 };
 
@@ -79,12 +87,22 @@ Arch linux package management utility
 =cut 
 
 sub pkg {
-    my $pkgref = shift;
-    my $type = shift;
-    my $pkg;
-    my $diff;
+    my ($pkg, $args) = @_;
+    # TODO detect assoc array
+    # If args is a hashref, deref and turn into an array, Csistck::Test
+    # processes arguments as named arguments -- an assoc array.
+    my $t_args = (ref($args) eq "HASH") ? [%{$args}] : [ 'type', $args ];
+    Csistck::Test::Pkg->new($pkg, @{$t_args});
+}
 
+sub new {
+    my $class = shift;
+    my $self = $class->SUPER::new(@_);
+    bless($self, $class);
+
+    # We'll fix package name here, instead of redoing each call
     # Priority: 'type' argument, 'pkg_type' option, detect_pkg_manager.
+    my $type = $self->{type};
     if (! $type) {
         $type = option('pkg_type') // detect_pkg_manager();
     }
@@ -92,21 +110,20 @@ sub pkg {
       if (! $type);
     die("Package manager not supported: type=<$type>")
       if (! $Cmds->{$type});
-    
-    $pkg = get_pkg($pkgref, $type);
-    $diff = sub { pkg_diff($pkg, $type); }
-      if ($Cmds->{$type}->{diff});
-    
-    return Csistck::Test->new(
-        check => sub { pkg_check($pkg, $type); },
-        repair => sub { pkg_install($pkg, $type); },
-        diff => $diff,
-        desc => "Searching for package $pkg, using $type"
-    );
+    $self->{type} = $type;
+
+    return $self;
 }
 
-sub pkg_check {
-    my ($pkg, $type) = @_;
+sub desc {
+    return sprintf("Package test for %s, using %s", 
+      $_[0]->pkg_name, $_[0]->pkg_type);
+}
+
+sub check {
+    my $self = shift;
+    my $pkg = $self->pkg_name;
+    my $type = $self->pkg_type;
     my $cmd = sprintf($Cmds->{$type}->{check}, $pkg) or
       die("Package check command missing: type=<$type>");
 
@@ -117,12 +134,19 @@ sub pkg_check {
       unless($ret == 0);
 }
 
-# Package install
-sub pkg_install {
-    my ($pkg, $type) = @_;
-    my $cmd = sprintf($Cmds->{$type}->{install}, $pkg) or
-      die("Package install command missing: type=<$type>");
-    
+sub repair {
+    my $self = shift;
+    my $pkg = $self->pkg_name;
+    my $type = $self->pkg_type;
+    my $cmd;
+
+    if (defined $Cmds->{$type}->{install}) {
+        $cmd = sprintf($Cmds->{$type}->{install}, $pkg);
+    }
+    else {
+        die("Package install command missing: type=<$type>");
+    }
+
     $ENV{DEBIAN_FRONTEND} = "noninteractive";
     debug("Installing package via command: cmd=<$cmd>");
     my $ret = system("$cmd 1>/dev/null 2>/dev/null");
@@ -132,10 +156,18 @@ sub pkg_install {
 }
 
 # Package diff
-sub pkg_diff {
-    my ($pkg, $type) = @_;
-    my $cmd = sprintf($Cmds->{$type}->{diff}, $pkg) or
-      die("Package diff command missing: type=<$type>");
+sub diff {
+    my $self = shift;
+    my $pkg = $self->pkg_name;
+    my $type = $self->pkg_type;
+    my $cmd;
+    
+    if (defined $Cmds->{$type}->{diff}) {
+        $cmd = sprintf($Cmds->{$type}->{diff}, $pkg);
+    }
+    else {
+        die("Package diff command missing: type=<$type>");
+    }
     
     $ENV{DEBIAN_FRONTEND} = "noninteractive";
     debug("Showing package differences via command: cmd=<$cmd>");
@@ -154,11 +186,12 @@ used for the package test.
 =cut
 
 sub detect_pkg_manager {
+    my $self = shift;
     given ("$^O") {
         when (/^freebsd$/) { return 'pkg_info'; }
         when (/^netbsd$/) { return 'pkg_info'; }
         when (/^linux$/) { 
-            given (distribution_name()) {
+            given (Linux::Distribution::distribution_name()) {
                 when (/^(?:debian|ubuntu)$/) { return 'dpkg'; }
                 when (/^(?:fedora|redhat|centos)$/) { return 'rpm'; }
                 when (/^gentoo$/) { return 'emerge'; }
@@ -171,7 +204,7 @@ sub detect_pkg_manager {
     }
 }
 
-=head2 get_pkg($package, $type)
+=head2 pkg_name($package, $type)
 
 Based on input package, return package name. With OS and distribution detection,
 $package can be passed as a string or a hashref.
@@ -180,13 +213,11 @@ See pkg() for information on passing hashrefs as package name
 
 =cut
 
-sub get_pkg {
-    my $pkg = shift;
-    my $type = shift;
-    
-    debug(sprintf('Searching for package: ref=<%s>, pkg=<%s>', 
-      ref $pkg, $pkg));
-    
+sub pkg_name {
+    my $self = shift;
+    my $pkg = $self->{target};
+    my $type = $self->{type};
+
     given (ref $pkg) {
         when ('') {
             return $pkg if ($pkg =~ m/^[A-Za-z0-9\-\_\.\/]+$/);
@@ -200,6 +231,14 @@ sub get_pkg {
     die('Invalid package');
 }
 
+=head2 pkg_type()
+
+Return fixed package type
+
+=cut
+
+sub pkg_type { $_[0]->{type}; }
+
 1;
 __END__
 
